package com.myoa.engineering.crawl.ppomppu.processor.service;

import java.util.List;

import org.springframework.stereotype.Service;

import com.myoa.engineering.crawl.ppomppu.processor.domain.PpomppuArticle;
import com.myoa.engineering.crawl.ppomppu.processor.dto.PpomppuArticleTransformer;
import com.myoa.engineering.crawl.ppomppu.processor.infrastructure.client.MessageSenderAPIClient;
import com.myoa.engineering.crawl.ppomppu.support.dto.code.PpomppuBoardName;

import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;

/**
 * MessageSenderService
 * @author Shin Woo-jin (woo-jin.shin@linecorp.com)
 * @since 2021-11-21
 *
 */
@Slf4j
@Service
public class MessageSenderService {

    private final MessageSenderAPIClient messageSenderAPIClient;

    public MessageSenderService(MessageSenderAPIClient messageSenderAPIClient) {
        this.messageSenderAPIClient = messageSenderAPIClient;
    }

    public Mono<String> sendSimpleMessageToSlack(PpomppuArticle article) {
        return messageSenderAPIClient.sendSimpleMessageToSlack(PpomppuArticleTransformer.TRANSFORM_TO_MESSAGE_DTO.apply(article));
    }

    public Mono<String> sendSimpleMessageToSlack(List<PpomppuArticle> articles) {
        return messageSenderAPIClient.sendSimpleMessageToSlack(PpomppuArticleTransformer.transformToSimpleMessage(articles));
    }

    public Mono<String> sendBlockMessageToSlack(PpomppuBoardName boardName, List<PpomppuArticle> articles) {
        return messageSenderAPIClient.sendBlockMessageToSlack(
            PpomppuArticleTransformer.transformToBlockMessage(boardName, articles));
    }

}
