package com.myoa.engineering.crawl.ppomppu.receiver.infrastructure.client;

import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.client.WebClientRequestException;

import com.myoa.engineering.crawl.ppomppu.receiver.dto.constant.WebClientPropertiesUnitName;
import com.myoa.engineering.crawl.ppomppu.support.dto.code.PpomppuBoardName;
import com.myoa.engineering.crawl.ppomppu.support.webclient.factory.WebClientFilterFactory;
import com.myoa.engineering.crawl.ppomppu.support.webclient.properties.WebClientProperties;
import com.myoa.engineering.crawl.ppomppu.support.webclient.properties.WebClientProperties.WebClientPropertiesUnit;

import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;
import reactor.core.scheduler.Schedulers;

@Slf4j
@Component
public class ProcessorAPIWebClient {

    private final WebClient webClient;

    public ProcessorAPIWebClient(WebClientProperties webClientProperties) {
        WebClientPropertiesUnit webClientPropertiesUnit =
            webClientProperties.find(WebClientPropertiesUnitName.PPOMPPU_NOTIFIER_PROCESSOR_API.getUnitName());
        this.webClient = WebClient.builder()
                                  .baseUrl(webClientPropertiesUnit.getBaseUrl())
                                  .filter(WebClientFilterFactory.logRequest())
                                  .filter(WebClientFilterFactory.logResponse())
                                  .build();
    }

    public Mono<String> emitParseEvent(PpomppuBoardName boardName) {
        return webClient.post()
                        .uri("/api/v1/crawl/boards/{boardName}", boardName)
                        .exchangeToMono(e -> e.bodyToMono(new ParameterizedTypeReference<String>() {}))
                        .publishOn(Schedulers.boundedElastic())
                        .onErrorResume(WebClientRequestException.class, t -> {
                            log.info("Exception occured, ignoring. : {}", t.getClass().getSimpleName());
                            return Mono.empty();
                        });
    }
}
